@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015-2016, Intel Corporation
+ * Copyright (c) 2015-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -27,10 +27,11 @@
  */
 
 /** \file
- * \brief NG, NGHolder, NGWrapper and graph handling.
+ * \brief NG and graph handling.
  */
-#include "grey.h"
 #include "ng.h"
+
+#include "grey.h"
 #include "ng_anchored_acyclic.h"
 #include "ng_anchored_dots.h"
 #include "ng_asserts.h"
@@ -41,6 +42,7 @@
 #include "ng_equivalence.h"
 #include "ng_extparam.h"
 #include "ng_fixed_width.h"
+#include "ng_fuzzy.h"
 #include "ng_haig.h"
 #include "ng_literal_component.h"
 #include "ng_literal_decorated.h"
@@ -52,7 +54,6 @@
 #include "ng_region.h"
 #include "ng_region_redundancy.h"
 #include "ng_reports.h"
-#include "ng_rose.h"
 #include "ng_sep.h"
 #include "ng_small_literal_set.h"
 #include "ng_som.h"
@@ -62,6 +63,7 @@
 #include "ng_util.h"
 #include "ng_width.h"
 #include "ue2common.h"
+#include "compiler/compiler.h"
 #include "nfa/goughcompile.h"
 #include "rose/rose_build.h"
 #include "smallwrite/smallwrite_build.h"
@@ -100,16 +102,16 @@ NG::~NG() {
  * \throw CompileError if SOM cannot be supported for the component.
  */
 static
-bool addComponentSom(NG &ng, NGHolder &g, const NGWrapper &w,
+bool addComponentSom(NG &ng, NGHolder &g, const ExpressionInfo &expr,
                      const som_type som, const u32 comp_id) {
     DEBUG_PRINTF("doing som\n");
-    dumpComponent(g, "03_presom", w.expressionIndex, comp_id, ng.cc.grey);
+    dumpComponent(g, "03_presom", expr.index, comp_id, ng.cc.grey);
     assert(hasCorrectlyNumberedVertices(g));
-    assert(allMatchStatesHaveReports(w));
+    assert(allMatchStatesHaveReports(g));
 
     // First, we try the "SOM chain" support in ng_som.cpp.
 
-    sombe_rv rv = doSom(ng, g, w, comp_id, som);
+    sombe_rv rv = doSom(ng, g, expr, comp_id, som);
     if (rv == SOMBE_HANDLED_INTERNAL) {
         return false;
     } else if (rv == SOMBE_HANDLED_ALL) {
@@ -118,7 +120,7 @@ bool addComponentSom(NG &ng, NGHolder &g, const NGWrapper &w,
     assert(rv == SOMBE_FAIL);
 
     /* Next, Sombe style approaches */
-    rv = doSomWithHaig(ng, g, w, comp_id, som);
+    rv = doSomWithHaig(ng, g, expr, comp_id, som);
     if (rv == SOMBE_HANDLED_INTERNAL) {
         return false;
     } else if (rv == SOMBE_HANDLED_ALL) {
@@ -132,7 +134,7 @@ bool addComponentSom(NG &ng, NGHolder &g, const NGWrapper &w,
     vector<vector<CharReach> > triggers; /* empty for outfix */
 
     assert(g.kind == NFA_OUTFIX);
-    dumpComponent(g, "haig", w.expressionIndex, comp_id, ng.cc.grey);
+    dumpComponent(g, "haig", expr.index, comp_id, ng.cc.grey);
     makeReportsSomPass(ng.rm, g);
     auto haig = attemptToBuildHaig(g, som, ng.ssm.somPrecision(), triggers,
                                    ng.cc.grey);
@@ -145,7 +147,7 @@ bool addComponentSom(NG &ng, NGHolder &g, const NGWrapper &w,
     /* Our various strategies for supporting SOM for this pattern have failed.
      * Provide a generic pattern not supported/too large return value as it is
      * unclear what the meaning of a specific SOM error would be */
-    throw CompileError(w.expressionIndex, "Pattern is too large.");
+    throw CompileError(expr.index, "Pattern is too large.");
 
     assert(0); // unreachable
     return false;
@@ -200,27 +202,35 @@ void reduceGraph(NGHolder &g, som_type som, bool utf8,
 }
 
 static
-bool addComponent(NG &ng, NGHolder &g, const NGWrapper &w, const som_type som,
-                  const u32 comp_id) {
+bool addComponent(NG &ng, NGHolder &g, const ExpressionInfo &expr,
+                  const som_type som, const u32 comp_id) {
     const CompileContext &cc = ng.cc;
     assert(hasCorrectlyNumberedVertices(g));
 
     DEBUG_PRINTF("expr=%u, comp=%u: %zu vertices, %zu edges\n",
-                 w.expressionIndex, comp_id, num_vertices(g), num_edges(g));
+                 expr.index, comp_id, num_vertices(g), num_edges(g));
 
-    dumpComponent(g, "01_begin", w.expressionIndex, comp_id, ng.cc.grey);
+    dumpComponent(g, "01_begin", expr.index, comp_id, ng.cc.grey);
 
-    assert(allMatchStatesHaveReports(w));
+    assert(allMatchStatesHaveReports(g));
 
-    reduceGraph(g, som, w.utf8, cc);
+    reduceExtendedParams(g, ng.rm, som);
+    reduceGraph(g, som, expr.utf8, cc);
 
-    dumpComponent(g, "02_reduced", w.expressionIndex, comp_id, ng.cc.grey);
+    dumpComponent(g, "02_reduced", expr.index, comp_id, ng.cc.grey);
 
     // There may be redundant regions that we can remove
     if (cc.grey.performGraphSimplification) {
         removeRegionRedundancy(g, som);
     }
 
+    // We might be done at this point: if we've run out of vertices, we can
+    // stop processing.
+    if (num_vertices(g) == N_SPECIALS) {
+        DEBUG_PRINTF("all vertices claimed\n");
+        return true;
+    }
+
     // "Short Exhaustible Passthrough" patterns always become outfixes.
     if (!som && isSEP(g, ng.rm, cc.grey)) {
         DEBUG_PRINTF("graph is SEP\n");
@@ -231,12 +241,12 @@ bool addComponent(NG &ng, NGHolder &g, const NGWrapper &w, const som_type som,
 
     // Start Of Match handling.
     if (som) {
-        if (addComponentSom(ng, g, w, som, comp_id)) {
+        if (addComponentSom(ng, g, expr, som, comp_id)) {
             return true;
         }
     }
 
-    assert(allMatchStatesHaveReports(w));
+    assert(allMatchStatesHaveReports(g));
 
     if (splitOffAnchoredAcyclic(*ng.rose, g, cc)) {
         return true;
@@ -251,15 +261,11 @@ bool addComponent(NG &ng, NGHolder &g, const NGWrapper &w, const som_type som,
         return true;
     }
 
-    if (doViolet(*ng.rose, g, w.prefilter, cc)) {
+    if (doViolet(*ng.rose, g, expr.prefilter, false, ng.rm, cc)) {
         return true;
     }
 
-    if (splitOffRose(*ng.rose, g, w.prefilter, cc)) {
-        return true;
-    }
-
-    if (splitOffPuffs(*ng.rose, ng.rm, g, w.prefilter, cc)) {
+    if (splitOffPuffs(*ng.rose, ng.rm, g, expr.prefilter, cc)) {
         return true;
     }
 
@@ -272,26 +278,7 @@ bool addComponent(NG &ng, NGHolder &g, const NGWrapper &w, const som_type som,
         return true;
     }
 
-    if (doViolet(*ng.rose, g, w.prefilter, cc)) {
-        return true;
-    }
-
-    if (splitOffRose(*ng.rose, g, w.prefilter, cc)) {
-        return true;
-    }
-
-    // A final pass at cyclic redundancy and Rose
-    // TODO: investigate - coverage results suggest that this never succeeds?
-    if (cc.grey.performGraphSimplification) {
-        if (removeCyclicPathRedundancy(g) ||
-            removeCyclicDominated(g, som)) {
-            if (handleFixedWidth(*ng.rose, g, cc.grey)) {
-                return true;
-            }
-        }
-    }
-
-    if (finalChanceRose(*ng.rose, g, w.prefilter, cc)) {
+    if (doViolet(*ng.rose, g, expr.prefilter, true, ng.rm, cc)) {
         return true;
     }
 
@@ -306,7 +293,7 @@ bool addComponent(NG &ng, NGHolder &g, const NGWrapper &w, const som_type som,
 
 // Returns true if all components have been added.
 static
-bool processComponents(NG &ng, NGWrapper &w,
+bool processComponents(NG &ng, ExpressionInfo &expr,
                        deque<unique_ptr<NGHolder>> &g_comp,
                        const som_type som) {
     const u32 num_components = g_comp.size();
@@ -316,7 +303,7 @@ bool processComponents(NG &ng, NGWrapper &w,
         if (!g_comp[i]) {
             continue;
         }
-        if (addComponent(ng, *g_comp[i], w, som, i)) {
+        if (addComponent(ng, *g_comp[i], expr, som, i)) {
             g_comp[i].reset();
             continue;
         }
@@ -336,40 +323,65 @@ bool processComponents(NG &ng, NGWrapper &w,
     return false;
 }
 
-bool NG::addGraph(NGWrapper &w) {
-    // remove reports that aren't on vertices connected to accept.
-    clearReports(w);
+bool NG::addGraph(ExpressionInfo &expr, unique_ptr<NGHolder> g_ptr) {
+    assert(g_ptr);
+    NGHolder &g = *g_ptr;
 
-    som_type som = w.som;
-    if (som && isVacuous(w)) {
-        throw CompileError(w.expressionIndex, "Start of match is not "
+    // remove reports that aren't on vertices connected to accept.
+    clearReports(g);
+
+    som_type som = expr.som;
+    if (som && isVacuous(g)) {
+        throw CompileError(expr.index, "Start of match is not "
                            "currently supported for patterns which match an "
                            "empty buffer.");
     }
 
-    dumpDotWrapper(w, "01_initial", cc.grey);
-    assert(allMatchStatesHaveReports(w));
+    dumpDotWrapper(g, expr, "01_initial", cc.grey);
+    assert(allMatchStatesHaveReports(g));
 
     /* ensure utf8 starts at cp boundary */
-    ensureCodePointStart(rm, w);
-    resolveAsserts(rm, w);
+    ensureCodePointStart(rm, g, expr);
 
-    dumpDotWrapper(w, "02_post_assert_resolve", cc.grey);
-    assert(allMatchStatesHaveReports(w));
-
-    pruneUseless(w);
-    pruneEmptyVertices(w);
-
-    if (can_never_match(w)) {
-        throw CompileError(w.expressionIndex, "Pattern can never match.");
+    if (can_never_match(g)) {
+        throw CompileError(expr.index, "Pattern can never match.");
     }
 
-    optimiseVirtualStarts(w); /* good for som */
+    // validate graph's suitability for fuzzing before resolving asserts
+    validate_fuzzy_compile(g, expr.edit_distance, expr.utf8, cc.grey);
 
-    handleExtendedParams(rm, w, cc);
-    if (w.min_length) {
-        // We have a minimum length constraint, which we currently use SOM to
-        // satisfy.
+    resolveAsserts(rm, g, expr);
+    dumpDotWrapper(g, expr, "02_post_assert_resolve", cc.grey);
+    assert(allMatchStatesHaveReports(g));
+
+    make_fuzzy(g, expr.edit_distance, cc.grey);
+    dumpDotWrapper(g, expr, "02a_post_fuzz", cc.grey);
+
+    pruneUseless(g);
+    pruneEmptyVertices(g);
+
+    if (can_never_match(g)) {
+        throw CompileError(expr.index, "Pattern can never match.");
+    }
+
+    optimiseVirtualStarts(g); /* good for som */
+
+    propagateExtendedParams(g, expr, rm);
+    reduceExtendedParams(g, rm, som);
+
+    // We may have removed all the edges to accept, in which case this
+    // expression cannot match.
+    if (can_never_match(g)) {
+        throw CompileError(expr.index, "Extended parameter constraints can not "
+                                       "be satisfied for any match from this "
+                                       "expression.");
+    }
+
+    if (any_of_in(all_reports(g), [&](ReportID id) {
+            return rm.getReport(id).minLength;
+        })) {
+        // We have at least one report with a minimum length constraint, which
+        // we currently use SOM to satisfy.
         som = SOM_LEFT;
         ssm.somPrecision(8);
     }
@@ -381,98 +393,104 @@ bool NG::addGraph(NGWrapper &w) {
     // first, we can perform graph work that can be done on an individual
     // expression basis.
 
-    if (w.utf8) {
-        relaxForbiddenUtf8(w);
+    if (expr.utf8) {
+        relaxForbiddenUtf8(g, expr);
     }
 
-    if (w.highlander && !w.min_length && !w.min_offset) {
+    if (all_of_in(all_reports(g), [&](ReportID id) {
+            const auto &report = rm.getReport(id);
+            return report.ekey != INVALID_EKEY && !report.minLength &&
+                   !report.minOffset;
+        })) {
         // In highlander mode: if we don't have constraints on our reports that
         // may prevent us accepting our first match (i.e. extended params) we
         // can prune the other out-edges of all vertices connected to accept.
-        pruneHighlanderAccepts(w, rm);
+        // TODO: shift the report checking down into pruneHighlanderAccepts()
+        // to allow us to handle the parts we can in mixed cases.
+        pruneHighlanderAccepts(g, rm);
     }
 
-    dumpDotWrapper(w, "02b_fairly_early", cc.grey);
+    dumpDotWrapper(g, expr, "02b_fairly_early", cc.grey);
 
     // If we're a vacuous pattern, we can handle this early.
-    if (splitOffVacuous(boundary, rm, w)) {
+    if (splitOffVacuous(boundary, rm, g, expr)) {
         DEBUG_PRINTF("split off vacuous\n");
     }
 
     // We might be done at this point: if we've run out of vertices, we can
     // stop processing.
-    if (num_vertices(w) == N_SPECIALS) {
+    if (num_vertices(g) == N_SPECIALS) {
         DEBUG_PRINTF("all vertices claimed by vacuous handling\n");
         return true;
     }
 
     // Now that vacuous edges have been removed, update the min width exclusive
     // of boundary reports.
-    minWidth = min(minWidth, findMinWidth(w));
+    minWidth = min(minWidth, findMinWidth(g));
 
     // Add the pattern to the small write builder.
-    smwr->add(w);
+    smwr->add(g, expr);
 
     if (!som) {
-        removeSiblingsOfStartDotStar(w);
+        removeSiblingsOfStartDotStar(g);
     }
 
-    dumpDotWrapper(w, "03_early", cc.grey);
+    dumpDotWrapper(g, expr, "03_early", cc.grey);
 
     // Perform a reduction pass to merge sibling character classes together.
     if (cc.grey.performGraphSimplification) {
-        removeRedundancy(w, som);
-        prunePathsRedundantWithSuccessorOfCyclics(w, som);
+        removeRedundancy(g, som);
+        prunePathsRedundantWithSuccessorOfCyclics(g, som);
     }
 
-    dumpDotWrapper(w, "04_reduced", cc.grey);
+    dumpDotWrapper(g, expr, "04_reduced", cc.grey);
 
     // If we've got some literals that span the graph from start to accept, we
     // can split them off into Rose from here.
     if (!som) {
-        if (splitOffLiterals(*this, w)) {
+        if (splitOffLiterals(*this, g)) {
             DEBUG_PRINTF("some vertices claimed by literals\n");
         }
     }
 
     // We might be done at this point: if we've run out of vertices, we can
     // stop processing.
-    if (num_vertices(w) == N_SPECIALS) {
+    if (num_vertices(g) == N_SPECIALS) {
         DEBUG_PRINTF("all vertices claimed before calc components\n");
         return true;
     }
 
-    // Split the graph into a set of connected components.
+    // Split the graph into a set of connected components and process those.
+    // Note: this invalidates g_ptr.
 
-    deque<unique_ptr<NGHolder>> g_comp = calcComponents(w);
+    auto g_comp = calcComponents(std::move(g_ptr), cc.grey);
     assert(!g_comp.empty());
 
     if (!som) {
-        for (u32 i = 0; i < g_comp.size(); i++) {
-            assert(g_comp[i]);
-            reformLeadingDots(*g_comp[i]);
+        for (auto &gc : g_comp) {
+            assert(gc);
+            reformLeadingDots(*gc);
         }
 
-        recalcComponents(g_comp);
+        recalcComponents(g_comp, cc.grey);
     }
 
-    if (processComponents(*this, w, g_comp, som)) {
+    if (processComponents(*this, expr, g_comp, som)) {
         return true;
     }
 
     // If we're in prefiltering mode, we can run the prefilter reductions and
     // have another shot at accepting the graph.
 
-    if (cc.grey.prefilterReductions && w.prefilter) {
-        for (u32 i = 0; i < g_comp.size(); i++) {
-            if (!g_comp[i]) {
+    if (cc.grey.prefilterReductions && expr.prefilter) {
+        for (auto &gc : g_comp) {
+            if (!gc) {
                 continue;
             }
-
-            prefilterReductions(*g_comp[i], cc);
+            prefilterReductions(*gc, cc);
         }
 
-        if (processComponents(*this, w, g_comp, som)) {
+        if (processComponents(*this, expr, g_comp, som)) {
             return true;
         }
     }
@@ -482,7 +500,7 @@ bool NG::addGraph(NGWrapper &w) {
         if (g_comp[i]) {
             DEBUG_PRINTF("could not compile component %u with %zu vertices\n",
                          i, num_vertices(*g_comp[i]));
-            throw CompileError(w.expressionIndex, "Pattern is too large.");
+            throw CompileError(expr.index, "Pattern is too large.");
         }
     }
 
@@ -491,63 +509,60 @@ bool NG::addGraph(NGWrapper &w) {
 }
 
 /** \brief Used from SOM mode to add an arbitrary NGHolder as an engine. */
-bool NG::addHolder(NGHolder &w) {
-    DEBUG_PRINTF("adding holder of %zu states\n", num_vertices(w));
-    assert(allMatchStatesHaveReports(w));
-    assert(hasCorrectlyNumberedVertices(w));
+bool NG::addHolder(NGHolder &g) {
+    DEBUG_PRINTF("adding holder of %zu states\n", num_vertices(g));
+    assert(allMatchStatesHaveReports(g));
+    assert(hasCorrectlyNumberedVertices(g));
 
     /* We don't update the global minWidth here as we care about the min width
      * of the whole pattern - not a just a prefix of it. */
 
     bool prefilter = false;
-    //dumpDotComp(comp, w, *this, 20, "prefix_init");
+    //dumpDotComp(comp, g, *this, 20, "prefix_init");
 
     som_type som = SOM_NONE; /* the prefixes created by the SOM code do not
                                 themselves track som */
     bool utf8 = false; // handling done earlier
-    reduceGraph(w, som, utf8, cc);
+    reduceGraph(g, som, utf8, cc);
 
     // There may be redundant regions that we can remove
     if (cc.grey.performGraphSimplification) {
-        removeRegionRedundancy(w, som);
+        removeRegionRedundancy(g, som);
     }
 
     // "Short Exhaustible Passthrough" patterns always become outfixes.
-    if (isSEP(w, rm, cc.grey)) {
+    if (isSEP(g, rm, cc.grey)) {
         DEBUG_PRINTF("graph is SEP\n");
-        if (rose->addOutfix(w)) {
+        if (rose->addOutfix(g)) {
             return true;
         }
     }
 
-    if (splitOffAnchoredAcyclic(*rose, w, cc)) {
+    if (splitOffAnchoredAcyclic(*rose, g, cc)) {
         return true;
     }
 
-    if (handleSmallLiteralSets(*rose, w, cc)
-        || handleFixedWidth(*rose, w, cc.grey)) {
+    if (handleSmallLiteralSets(*rose, g, cc)
+        || handleFixedWidth(*rose, g, cc.grey)) {
         return true;
     }
 
-    if (handleDecoratedLiterals(*rose, w, cc)) {
+    if (handleDecoratedLiterals(*rose, g, cc)) {
         return true;
     }
 
-    if (splitOffRose(*rose, w, prefilter, cc)) {
+    if (doViolet(*rose, g, prefilter, false, rm, cc)) {
         return true;
     }
-    if (splitOffPuffs(*rose, rm, w, prefilter, cc)) {
+    if (splitOffPuffs(*rose, rm, g, prefilter, cc)) {
         return true;
     }
-    if (splitOffRose(*rose, w, prefilter, cc)) {
-        return true;
-    }
-    if (finalChanceRose(*rose, w, prefilter, cc)) {
+    if (doViolet(*rose, g, prefilter, true, rm, cc)) {
         return true;
     }
 
     DEBUG_PRINTF("trying for outfix\n");
-    if (rose->addOutfix(w)) {
+    if (rose->addOutfix(g)) {
         DEBUG_PRINTF("ok\n");
         return true;
     }
@@ -602,24 +617,4 @@ bool NG::addLiteral(const ue2_literal &literal, u32 expr_index,
     return true;
 }
 
-NGWrapper::NGWrapper(unsigned int ei, bool highlander_in, bool utf8_in,
-                     bool prefilter_in, som_type som_in, ReportID r,
-                     u64a min_offset_in, u64a max_offset_in, u64a min_length_in)
-    : expressionIndex(ei), reportId(r), highlander(highlander_in),
-      utf8(utf8_in), prefilter(prefilter_in), som(som_in),
-      min_offset(min_offset_in), max_offset(max_offset_in),
-      min_length(min_length_in) {
-    // All special nodes/edges are added in NGHolder's constructor.
-    DEBUG_PRINTF("built %p: expr=%u report=%u%s%s%s%s "
-                 "min_offset=%llu max_offset=%llu min_length=%llu\n",
-                 this, expressionIndex, reportId,
-                 highlander ? " highlander" : "",
-                 utf8 ? " utf8" : "",
-                 prefilter ? " prefilter" : "",
-                 (som != SOM_NONE) ? " som" : "",
-                 min_offset, max_offset, min_length);
-}
-
-NGWrapper::~NGWrapper() {}
-
 } // namespace ue2
