@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Intel Corporation
+ * Copyright (c) 2016-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -38,5 +38,7 @@ extern bool saveDatabases;
 extern bool loadDatabases;
 extern std::string serializePath;
 extern unsigned int somPrecisionMode;
+extern bool forceEditDistance;
+extern unsigned editDistance;
 
 #endif // COMMON_H
